from .experiment import *
from .emu_emu_wireless import EmuEmuWireless
from .emu_phy_wireless import EmuPhyWireless


emuEmu = EmuEmuWireless()
emuEmu.setup()
runThroughput(EMU_EMU_WIRELESS_PREFIX, EMU_EMU_WIRELESS_EPC_IP_ADDR, EMU_EMU_WIRELESS_UE_IP_ADDR)
runRTT(EMU_EMU_WIRELESS_PREFIX, EMU_EMU_WIRELESS_EPC_IP_ADDR, EMU_EMU_WIRELESS_UE_IP_ADDR)
runLatency(EMU_EMU_WIRELESS_PREFIX, EMU_EMU_WIRELESS_EPC_IP_ADDR, EMU_EMU_WIRELESS_UE_IP_ADDR)
emuEmu.tearDown()

emuPhy = EmuPhyWireless()
emuPhy.setup()
runThroughput(EMU_PHY_WIRELESS_PREFIX, EMU_PHY_WIRELESS_EPC_IP_ADDR, EMU_PHY_WIRELESS_UE_IP_ADDR)
runRTT(EMU_PHY_WIRELESS_PREFIX, EMU_PHY_WIRELESS_EPC_IP_ADDR, EMU_PHY_WIRELESS_UE_IP_ADDR)
runLatency(EMU_PHY_WIRELESS_PREFIX, EMU_PHY_WIRELESS_EPC_IP_ADDR, EMU_PHY_WIRELESS_UE_IP_ADDR)
emuPhy.tearDown()

runThroughput(PHY_PHY_WIRELESS_PREFIX, PHY_PHY_WIRELESS_EPC_IP_ADDR, PHY_PHY_WIRELESS_UE_IP_ADDR)
runRTT(PHY_PHY_WIRELESS_PREFIX, PHY_PHY_WIRELESS_EPC_IP_ADDR, PHY_PHY_WIRELESS_UE_IP_ADDR)
runLatency(PHY_PHY_WIRELESS_PREFIX, PHY_PHY_WIRELESS_EPC_IP_ADDR, PHY_PHY_WIRELESS_UE_IP_ADDR)

    